package Image::Randim;
our $VERSION = '0.01';
use v5.20;

1;

=pod

=head1 NAME

Image::Randim - Placeholder for Image::Randim::Source (for now)

=head1 DESCRIPTION

For now, this module just holds a place in the module tree to keep
systems happy that expect a module to exist at the root distribution
name.

=cut
